WeightStation = {};

function WeightStation.prerequisitesPresent(specializations)
    return true;
end;

function WeightStation:load(xmlFile)

	self.setDigitNumbers = SpecializationUtil.callSpecializationsFunction("setDigitNumbers");
	self.getVehicleMass = SpecializationUtil.callSpecializationsFunction("getVehicleMass");
	self.onVehicleCallback = WeightStation.onVehicleCallback;
	self.setClockTime = SpecializationUtil.callSpecializationsFunction("setClockTime");
	self.rotateClock = SpecializationUtil.callSpecializationsFunction("rotateClock");
	self.groundContactReport = SpecializationUtil.callSpecializationsFunction("groundContactReport");
	self.setBuildState = SpecializationUtil.callSpecializationsFunction("setBuildState");
	self.cultivateArea = SpecializationUtil.callSpecializationsFunction("cultivateArea");

	self.buildSet = {};
	self.buildSet.offset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.build#offset"), 0.0);
	self.buildSet.isBuild = false;
	self.buildSet.isVisibility = false;
	self:setBuildState(self.buildSet.isBuild, self.buildSet.isVisibility, true);
	self.buildSet.areaStart = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.cultivatorArea#start"));
	self.buildSet.areaWidth = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.cultivatorArea#width"));
	self.buildSet.areaHeight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.cultivatorArea#height"));
	self.buildSet.isRigidBodyTypeSet = false;
	self.buildSet.time = 0;
	
	self.fruitHectoliters = {};
	local i=0;
	while true do
		local key = string.format("vehicle.fruitHectoliters.fruit(%d)", i);
        local fruitNameStr = getXMLString(xmlFile, key .. "#name");
        if fruitNameStr == nil then
            break;
        end;
		self.fruitHectoliters[Fillable.fillTypeNameToInt[fruitNameStr]] = {};
		self.fruitHectoliters[Fillable.fillTypeNameToInt[fruitNameStr]].hectoliter = getXMLFloat(xmlFile, key .. "#hectoliter")/100;
		i=i+1;
	end;
	
	self.clock = {};
	self.clock.index = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.clockSets#index"));
	self.clock.speed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.build.clockSets#speed"), 1)/10000;
	self.clock.front = {};
	self.clock.front.hour = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.clockSetFront#hour"));
	self.clock.front.minute = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.clockSetFront#minute"));
	self.clock.front.invert = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.build.clockSetFront#invert"), false);
	self.clock.back = {};
	self.clock.back.hour = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.clockSetBack#hour"));
	self.clock.back.minute = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.clockSetBack#minute"));
	self.clock.back.invert = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.build.clockSetBack#invert"), false);
	
	self.weightStation = {};
	self.weightStation.trigger = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.weightTrigger#index"));
	addTrigger(self.weightStation.trigger, "onVehicleCallback", self);
	self.weightStation.isEnabled = false;
	self.weightStation.digits = {};
	self.weightStation.digits[1] = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.digits#digit1"));
	self.weightStation.digits[2] = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.digits#digit2"));
	self.weightStation.digits[3] = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.digits#digit3"));
	self.weightStation.digits[4] = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.build.digits#digit4"));
	
	self.weightStation.digitPosition = {};
	self.weightStation.digitPosition["0"] = -1.000;
	self.weightStation.digitPosition["1"] = -0.105;
	self.weightStation.digitPosition["2"] = -0.208;
	self.weightStation.digitPosition["3"] = -0.306;
	self.weightStation.digitPosition["4"] = -0.410;
	self.weightStation.digitPosition["5"] = -0.517;
	self.weightStation.digitPosition["6"] = -0.613;
	self.weightStation.digitPosition["7"] = -0.719;
	self.weightStation.digitPosition["8"] = -0.817;
	self.weightStation.digitPosition["9"] = -0.926;
	
	self.terrainContactNode = {};
	self.terrainContactNode.node = self.components[1].node;
	self.terrainContactNode.hasGroundContact = false;
	self.terrainContactNode.contactReportActive = false;
	
	self.vehiclesInTrigger = {};
	self.allVehicleMass = 0;
	self.noRenderOnLoad = false;
	self.allowBuild = true;
	self.playerToWeight = {};
	self.balesToWeight = {};
end;

function WeightStation:delete()
	WeightStation.removeContactReports(self);
	if self.weightStation.trigger ~= nil then
		removeTrigger(self.weightStation.trigger);
	end;
end;

function WeightStation:readStream(streamId, connection)
	if connection:getIsServer() then
		local buildState = streamReadBool(streamId);
		local isVisibility = streamReadBool(streamId);
		self:setBuildState(buildState, isVisibility, true);
	end;
end;

function WeightStation:writeStream(streamId, connection)
	if not connection:getIsServer() then
		streamWriteBool(streamId, self.buildSet.isBuild);
		streamWriteBool(streamId, self.buildSet.isVisibility);
	end;
end;

function WeightStation:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WeightStation:keyEvent(unicode, sym, modifier, isDown)
end;

function WeightStation:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicle then
		local isBuild = getXMLString(xmlFile, key.."#isBuild");
		if isBuild == "true" then
			self:setBuildState(true, true);
			self.noRenderOnLoad = true;
			self.terrainContactNode.hasGroundContact = true;
			self.terrainContactNode.contactReportActive = true;
		end;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function WeightStation:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'isBuild="'..tostring(self.buildSet.isBuild)..'"';
	return attributes, nil;
end;

function WeightStation:update(dt)

	local otherStationAllow = false;
	for k, vehicle in pairs(g_currentMission.vehicles) do
		if vehicle ~= self then
			if vehicle.buildSet ~= nil and vehicle.allowBuild and not vehicle.buildSet.isRigidBodyTypeSet then
				otherStationAllow = true;
			end;
		elseif vehicle == self then
			if otherStationAllow then
				self.allowBuild = false;
				break;
			elseif not otherStationAllow then
				self.allowBuild = true;
				break;
			end;
		end;
	end;
	
	if self.isServer then
		if self.allowBuild then
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) and g_currentMission.player.isControlled and g_currentMission.player.isEntered then
				if not self.buildSet.isBuild then
					self:setBuildState(self.buildSet.isBuild, not self.buildSet.isVisibility);
				end;
			end;
			if self.buildSet.isVisibility then
				if not self.buildSet.isBuild then
					if not g_currentMission.player.isControlled and not g_currentMission.player.isEntered then
						self:setBuildState(self.buildSet.isBuild, false);
					end;
					g_currentMission:addExtraPrintText(g_i18n:getText("helpText"));
					g_currentMission:addHelpButtonText(g_i18n:getText("BuildSet_2"), InputBinding.IMPLEMENT_EXTRA);
					g_currentMission:addHelpButtonText(g_i18n:getText("BuildSet_3"), InputBinding.ACTIVATE_OBJECT);
					
					link(g_currentMission.player.graphicsRootNode, self.components[1].node)
					setTranslation(self.components[1].node, 0, 0, -20);
					setRotation(self.components[1].node, 0, 0, 0);
				
					local x, y, z = getWorldTranslation(self.components[1].node);
					local terrainHeight = y;
					local xRot, yRot, zRot = getWorldRotation(self.components[1].node);
					link(getRootNode(), self.components[1].node);
					setTranslation(self.components[1].node, x, y+self.buildSet.offset, z);
					setRotation(self.components[1].node, xRot, yRot, zRot);
					
					local buildFinish = false;
					if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
						buildFinish = true;
					end;
					self:setBuildState(buildFinish, self.buildSet.isVisibility);
				end;
			elseif g_currentMission.player.isControlled and g_currentMission.player.isEntered then
				g_currentMission:addHelpButtonText(g_i18n:getText("BuildSet_1"), InputBinding.IMPLEMENT_EXTRA);
			end;
		elseif not self.buildSet.isBuild then
			self:setBuildState(self.buildSet.isBuild, false);
		end;		
	end;

	self.weightStation.isEnabled = self.buildSet.isRigidBodyTypeSet and self.buildSet.isBuild;
	
	if self.weightStation.isEnabled then
		self:getVehicleMass(dt);
		self:setClockTime();
		self:rotateClock(dt);
	end;
	
	local bodyType = getRigidBodyType(self.components[1].node);
	if bodyType == "Dynamic" then
		local worldX, worldY, worldZ = localDirectionToWorld(self.components[1].node, 0, -dt*10, 0);
		addForce(self.components[1].node, worldX, worldY, worldZ, 0, 0, 0, true);
		if self.isServer and not self.noRenderOnLoad then
			g_currentMission:addWarning(g_i18n:getText("buildWait"), 0.040, 0.035);
		end;
	end;
end;

function WeightStation:setDigitNumbers(mass)
	local mass = mass;
	local massUnits = string.format("%d", mass);
	local digit1 = "0";
	local digit2 = "0";
	local digit3 = "0";
	local digit4 = "0"
	if mass >= 10 then
		digit4 = string.sub(massUnits, 1, 1);
	end;
	if mass >= 100 then
		digit3 = string.sub(massUnits, 1, 1);
		digit4 = string.sub(massUnits, 2, 2);
	end;
	if mass >= 1000 then
		digit2 = string.sub(massUnits, 1, 1);
		digit3 = string.sub(massUnits, 2, 2);
		digit4 = string.sub(massUnits, 3, 3);
	end;
	if mass >= 10000 then
		digit1 = string.sub(massUnits, 1, 1);
		digit2 = string.sub(massUnits, 2, 2);
		digit3 = string.sub(massUnits, 3, 3);
		digit4 = string.sub(massUnits, 4, 4);
	end;
	setShaderParameter(self.weightStation.digits[4], "Position", self.weightStation.digitPosition[digit4], 0, 0, 0, false);
	setShaderParameter(self.weightStation.digits[3], "Position", self.weightStation.digitPosition[digit3], 0, 0, 0, false);
	setShaderParameter(self.weightStation.digits[2], "Position", self.weightStation.digitPosition[digit2], 0, 0, 0, false);
	setShaderParameter(self.weightStation.digits[1], "Position", self.weightStation.digitPosition[digit1], 0, 0, 0, false);
end;

function WeightStation:setClockTime()
	local dayTime = Utils.getNoNil(g_currentMission.environment.dayTime, 0);
	local timeHours = dayTime/(1000*60*60);
	local timeHours2 = math.floor(timeHours);
    local timeMinutes = math.floor(((dayTime/(1000*60*60))-timeHours2)*60);
	local hourRotation = (360/12)*timeHours;
	if hourRotation > 360 then
		hourRotation = math.abs(hourRotation-360);
	end;
	local minuteRotation = (360/60)*timeMinutes;
	if self.clock.front.invert then
		setRotation(self.clock.front.hour, 0, 0, Utils.degToRad(hourRotation));
		setRotation(self.clock.front.minute, 0, 0, Utils.degToRad(minuteRotation));
	else
		setRotation(self.clock.front.hour, 0, 0, Utils.degToRad(hourRotation*-1));
		setRotation(self.clock.front.minute, 0, 0, Utils.degToRad(minuteRotation*-1));
	end;
	if self.clock.back.invert then
		setRotation(self.clock.back.hour, 0, 0, Utils.degToRad(hourRotation));
		setRotation(self.clock.back.minute, 0, 0, Utils.degToRad(minuteRotation));
	else
		setRotation(self.clock.back.hour, 0, 0, Utils.degToRad(hourRotation*-1));
		setRotation(self.clock.back.minute, 0, 0, Utils.degToRad(minuteRotation*-1));
	end;
end;

function WeightStation:rotateClock(dt)
	rotate(self.clock.index, 0, dt*self.clock.speed, 0);
end;

function WeightStation:onVehicleCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if self.weightStation.isEnabled then
		if onStay then
			local doInsertVehicle = true;
			local doInsertPlayer = true;
			local doInsertBale = true;
			local vehicle = g_currentMission.nodeToVehicle[otherId];
			for k, vehicles in pairs(self.vehiclesInTrigger) do
				if vehicle == vehicles then
					doInsertVehicle = false;
				end;
			end;
			for k, player in pairs(self.playerToWeight) do
				if otherId == player.rootNode then
					doInsertPlayer = false;
				end;
			end;
			for k, bale in pairs(self.balesToWeight) do
				if otherId == bale.nodeId then
					doInsertBale = false;
				end;
			end;
			if doInsertVehicle then
				table.insert(self.vehiclesInTrigger, vehicle);
			end;
			if doInsertPlayer then
				for i, misssionPlayers in pairs(g_currentMission.players) do
					if otherId == misssionPlayers.rootNode then
						table.insert(self.playerToWeight, misssionPlayers);
					end;
				end;
			end;
			if doInsertBale then
				for index, item in pairs(g_currentMission.itemsToSave) do
					if item.item:isa(Bale) then
						if item.item.nodeId == otherId then
							table.insert(self.balesToWeight, item.item);
						end;
					end;
				end;
			end;
		elseif onLeave then
			local vehicle = g_currentMission.nodeToVehicle[otherId];
			for k, v in pairs(self.vehiclesInTrigger) do
				if vehicle == v then
					table.remove(self.vehiclesInTrigger, k);
				end;
			end;
			for k, player in pairs(self.playerToWeight) do
				if player.rootNode == otherId then
					table.remove(self.playerToWeight, k);
				end;
			end;
			for k, bale in pairs(self.balesToWeight) do
				if bale.nodeId == otherId then
					table.remove(self.balesToWeight, k);
				end;
			end;
		end;
	end;
end;

function WeightStation:updateTick(dt)
	if self.buildSet.isBuild then
		if not self.buildSet.isRigidBodyTypeSet and self.terrainContactNode.hasGroundContact then
			if self.buildSet.time < 1000 then
				self.buildSet.time = self.buildSet.time + dt;
			else
				setRigidBodyType(self.components[1].node, "Static");
				setCollisionMask(self.components[1].node, 2102);
				self.buildSet.isRigidBodyTypeSet = true;
				WeightStation.removeContactReports(self);
			end;
		end;
	else
		self.buildSet.time = 0;
	end;
end;

function WeightStation:setBuildState(buildState, buildVisibility, noEventSend)
	SetBuildState.sendEvent(self, buildState, buildVisibility, noEventSend);
	self.buildSet.isBuild = buildState;
	self.buildSet.isVisibility = buildVisibility;
	setVisibility(self.components[1].node, self.buildSet.isVisibility);
	if self.buildSet.isBuild == true then
		setRigidBodyType(self.components[1].node, "Dynamic");
		setCollisionMask(self.components[1].node, 2102);
		self.buildSet.isRigidBodyTypeSet = false;
		WeightStation.addContactReports(self);
		self:cultivateArea();
	elseif self.buildSet.isBuild == false then
		setRigidBodyType(self.components[1].node ,"NoRigidBody");
	end;
end;

function WeightStation:cultivateArea()
	local x, y, z = getWorldTranslation(self.buildSet.areaStart);
	local x1, y1, z1 = getWorldTranslation(self.buildSet.areaWidth);
	local x2, y2, z2 = getWorldTranslation(self.buildSet.areaHeight);
	local detailId = g_currentMission.terrainDetailId;
	Utils.getXZWidthAndHeight(detailId, x, z, x1, z1, x2, z2);
	Utils.updateDestroyCommonArea(x, z, x1, z1, x2, z2);
end;

function WeightStation:getVehicleMass(dt)
	local mass = 0;
	for k, vehicle in pairs(self.vehiclesInTrigger) do
		for k, component in pairs(vehicle.components) do
			mass = mass + getMass(component.node);	
		end;
	end;
	mass = mass*1000;
	for k, vehicle in pairs(self.vehiclesInTrigger) do
		if vehicle.fillLevelMax == nil then
			if vehicle.fillLevel ~= nil then
				mass = mass-(vehicle.fillLevel*vehicle.massScale);
				local massScale = 0.70;
				for k, fruit in pairs(self.fruitHectoliters) do
					if k == vehicle.currentFillType and fruit then
						massScale = fruit.hectoliter;
					end;
				end;
				mass = mass+(vehicle.fillLevel*massScale);
			end;
			if vehicle.grainTankFillLevel ~= nil then
				local massScale = 0.70;
				for k, fruit in pairs(self.fruitHectoliters) do
					if k == vehicle.currentGrainTankFruitType and fruit then
						massScale = fruit.hectoliter;
					end;
				end;
				mass = mass+(vehicle.grainTankFillLevel*massScale);
			end;
		elseif vehicle.fillLevelMax ~= nil then
			mass = mass+(633*vehicle.fillLevel);
		end;
		if vehicle.fuelFillLevel ~= nil then
			mass = mass+vehicle.fuelFillLevel;
		end;
		if vehicle.isControlled then
			mass = mass+80;
		end;
	end;
	local numPlayers = table.getn(self.playerToWeight);
	mass = mass+(80*numPlayers);
	local numBales = table.getn(self.balesToWeight);
	mass = mass+(633*numBales);
	if mass > self.allVehicleMass then
		self.allVehicleMass = math.min(self.allVehicleMass+(dt*5), mass);
	elseif mass < self.allVehicleMass then
		self.allVehicleMass = math.max(self.allVehicleMass-(dt*15), mass);
	end;
	self:setDigitNumbers(self.allVehicleMass);
end;

function WeightStation:addContactReports()
	if not self.terrainContactNode.contactReportActive then
		if self.terrainContactNode.node ~= nil then
			addContactReport(self.terrainContactNode.node, 0.0001, "groundContactReport", self);
			self.terrainContactNode.contactReportActive = true;
		end;
	end;
end;

function WeightStation:removeContactReports()
	if self.terrainContactNode.contactReportActive then
		if self.terrainContactNode.node ~= nil then
			removeContactReport(self.terrainContactNode.node);
			self.terrainContactNode.hasGroundContact = false;
			self.terrainContactNode.contactReportActive = false;
		end;
	end;
end;

function WeightStation:groundContactReport(objectId, otherObjectId, isStart, normalForce, tangentialForce)
	if otherObjectId ~= self then
		if self.terrainContactNode.node ~= nil then
			if not self.terrainContactNode.hasGroundContact and self.buildSet.time < 10 then
				self.terrainContactNode.hasGroundContact = isStart or normalForce > 0 or tangentialForce > 0;
			end;
		end;
	end;
end;

function WeightStation:draw()	
end;